@@ -27,7 +27,6 @@
 import atexit
 import logging
 import os
-import shutil
 import time
 
 import gpodder
@@ -91,6 +90,8 @@ defaults = {
         'retries': 3,  # number of retries when downloads time out
     },
 
+    'check_connection': True,
+
     # Software updates from gpodder.org
     'software_update': {
         'check_on_startup': True,  # check for updates on start
@@ -162,6 +163,7 @@ defaults = {
                 'descriptions': True,
                 'view_mode': 1,
                 'columns': int('110', 2),  # bitfield of visible columns
+                'always_show_new': True,
             },
 
             'download_list': {
@@ -203,6 +205,8 @@ defaults = {
     'youtube': {
         'preferred_fmt_id': 18,  # default fmt_id (see fallbacks in youtube.py)
         'preferred_fmt_ids': [],  # for advanced uses (custom fallback sequence)
+        'preferred_hls_fmt_id': 93,  # default fmt_id (see fallbacks in youtube.py)
+        'preferred_hls_fmt_ids': [],  # for advanced uses (custom fallback sequence)
     },
 
     'vimeo': {
@@ -216,8 +220,6 @@ defaults = {
 
 # The sooner this goes away, the better
 gPodderSettings_LegacySupport = {
-    'player': 'player.audio',
-    'videoplayer': 'player.video',
     'limit_rate': 'limit.bandwidth.enabled',
     'limit_rate_value': 'limit.bandwidth.kbps',
     'max_downloads_enabled': 'limit.downloads.enabled',
@@ -344,14 +346,17 @@ class Config(object):
         logger.info('Flushing settings to disk')
 
         try:
-            fp = open(filename + '.tmp', 'wt')
-            fp.write(repr(self.__json_config))
-            fp.close()
+            # revoke unix group/world permissions (this has no effect under windows)
+            umask = os.umask(0o077)
+            with open(filename + '.tmp', 'wt') as fp:
+                fp.write(repr(self.__json_config))
             util.atomic_rename(filename + '.tmp', filename)
         except:
             logger.error('Cannot write settings to %s', filename)
             util.delete_file(filename + '.tmp')
             raise
+        finally:
+            os.umask(umask)
 
         self.__save_thread = None
 
