@@ -36,40 +36,51 @@ import socket
 import threading
 import time
 import urllib.error
-import urllib.parse
-import urllib.request
-from email.header import decode_header
+
+import requests
+from requests.adapters import HTTPAdapter
+from requests.exceptions import ConnectionError, HTTPError, RequestException
+from requests.packages.urllib3.exceptions import MaxRetryError
+from requests.packages.urllib3.util.retry import Retry
 
 import gpodder
-from gpodder import escapist_videos, util, vimeo, youtube
+from gpodder import registry, util
 
 logger = logging.getLogger(__name__)
 
 _ = gpodder.gettext
 
+REDIRECT_RETRIES = 3
 
-def get_header_param(headers, param, header_name):
-    """Extract a HTTP header parameter from a dict
 
-    Uses the "email" module to retrieve parameters
-    from HTTP headers. This can be used to get the
-    "filename" parameter of the "content-disposition"
-    header for downloads to pick a good filename.
+class CustomDownload:
+    """ abstract class for custom downloads. DownloadTask call retrieve_resume() on it """
 
-    Returns None if the filename cannot be retrieved.
+    def retrieve_resume(self, tempname, reporthook):
+        """
+        :param str tempname: temporary filename for the download
+        :param func(number, number, number) reporthook: callback for download progress (count, blockSize, totalSize)
+        :return dict(str, str), str: (headers, real_url)
+        """
+        return {}, None
+
+
+class CustomDownloader:
     """
-    value = None
-    try:
-        headers_string = ['%s:%s' % (k, v) for k, v in list(headers.items())]
-        msg = email.message_from_string('\n'.join(headers_string))
-        if header_name in msg:
-            raw_value = msg.get_param(param, header=header_name)
-            if raw_value is not None:
-                value = email.utils.collapse_rfc2231_value(raw_value)
-    except Exception as e:
-        logger.error('Cannot get %s from %s', param, header_name, exc_info=True)
+    abstract class for custom downloaders.
 
-    return value
+    DownloadTask calls custom_downloader to get a CustomDownload
+    """
+
+    def custom_downloader(self, config, episode):
+        """
+        if this custom downloader has a custom download method (e.g. youtube-dl),
+        return a CustomDownload. Else return None
+        :param config: gpodder config (e.g. to get preferred video format)
+        :param model.PodcastEpisode episode: episode to download
+        :return CustomDownload: object used to download the episode
+        """
+        return None
 
 
 class ContentRange(object):
@@ -177,7 +188,7 @@ class ContentRange(object):
 class DownloadCancelledException(Exception): pass
 
 
-class AuthenticationError(Exception): pass
+class DownloadNoURLException(Exception): pass
 
 
 class gPodderDownloadHTTPError(Exception):
@@ -187,74 +198,38 @@ class gPodderDownloadHTTPError(Exception):
         self.error_message = error_message
 
 
-class DownloadURLOpener(urllib.request.FancyURLopener):
-    version = gpodder.user_agent
+class DownloadURLOpener:
 
     # Sometimes URLs are not escaped correctly - try to fix them
     # (see RFC2396; Section 2.4.3. Excluded US-ASCII Characters)
     # FYI: The omission of "%" in the list is to avoid double escaping!
     ESCAPE_CHARS = dict((ord(c), '%%%x' % ord(c)) for c in ' <>#"{}|\\^[]`')
 
-    def __init__(self, channel):
-        self.channel = channel
-        self._auth_retry_counter = 0
+    def __init__(self, channel, max_retries=3):
         super().__init__()
+        self.channel = channel
+        self.max_retries = max_retries
 
-    def http_error(self, url, fp, errcode, errmsg, headers, data=None):
-        """Handle http errors.
-        Overriden to give retry=True to http_error_40{1,7}.
-        See https://github.com/python/cpython/commit/80f1b059714aeb1c6fc9f6ce1173bc8a51af7dd9
-        See python issue https://bugs.python.org/issue1368368
-        """
-        result = False
-        if errcode == 401:
-            result = self.http_error_401(url, fp, errcode, errmsg, headers, data=data, retry=True)
-        elif errcode == 407:
-            result = self.http_error_407(url, fp, errcode, errmsg, headers, data=data, retry=True)
-        if result:
-            return result
-        return super().http_error(url, fp, errcode, errmsg, headers, data=data)
-
-    def http_error_default(self, url, fp, errcode, errmsg, headers):
-        """
-        FancyURLopener by default does not raise an exception when
-        there is some unknown HTTP error code. We want to override
-        this and provide a function to log the error and raise an
-        exception, so we don't download the HTTP error page here.
-        """
-        # The following two lines are copied from urllib.URLopener's
-        # implementation of http_error_default
-        void = fp.read()
-        fp.close()
-        raise gPodderDownloadHTTPError(url, errcode, errmsg)
-
-    def redirect_internal(self, url, fp, errcode, errmsg, headers, data):
-        """ This is the exact same function that's included with urllib
-            except with "void = fp.read()" commented out. """
-
-        if 'location' in headers:
-            newurl = headers['location']
-        elif 'uri' in headers:
-            newurl = headers['uri']
-        else:
-            return
-
-        # This blocks forever(?) with certain servers (see bug #465)
-        # void = fp.read()
-        fp.close()
-
-        # In case the server sent a relative URL, join with original:
-        newurl = urllib.parse.urljoin(self.type + ":" + url, newurl)
-        return self.open(newurl)
+    def init_session(self):
+        """ init a session with our own retry codes + retry count """
+        # I add a few retries for redirects but it means that I will allow max_retries + REDIRECT_RETRIES
+        # if encountering max_retries connect and REDIRECT_RETRIES read for instance
+        retry_strategy = Retry(
+            total=self.max_retries + REDIRECT_RETRIES,
+            connect=self.max_retries,
+            read=self.max_retries,
+            redirect=max(REDIRECT_RETRIES, self.max_retries),
+            status=self.max_retries,
+            status_forcelist=Retry.RETRY_AFTER_STATUS_CODES.union((408, 418, 504, 598, 599,)))
+        adapter = HTTPAdapter(max_retries=retry_strategy)
+        http = requests.Session()
+        http.mount("https://", adapter)
+        http.mount("http://", adapter)
+        return http
 
 # The following is based on Python's urllib.py "URLopener.retrieve"
 # Also based on http://mail.python.org/pipermail/python-list/2001-October/110069.html
 
-    def http_error_206(self, url, fp, errcode, errmsg, headers, data=None):
-        # The next line is taken from urllib's URLopener.open_http
-        # method, at the end after the line "if errcode == 200:"
-        return urllib.addinfourl(fp, headers, 'http:' + url)
-
     def retrieve_resume(self, url, filename, reporthook=None, data=None):
         """Download files from an URL; return (headers, real_url)
 
@@ -264,13 +239,23 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
 
         current_size = 0
         tfp = None
+        headers = {
+            'User-agent': gpodder.user_agent
+        }
+
+        if self.channel.auth_username or self.channel.auth_password:
+            logger.debug('Authenticating as "%s"', self.channel.auth_username)
+            auth = (self.channel.auth_username, self.channel.auth_password)
+        else:
+            auth = None
+
         if os.path.exists(filename):
             try:
                 current_size = os.path.getsize(filename)
                 tfp = open(filename, 'ab')
                 # If the file exists, then only download the remainder
                 if current_size > 0:
-                    self.addheader('Range', 'bytes=%s-' % (current_size))
+                    headers['Range'] = 'bytes=%s-' % (current_size)
             except:
                 logger.warn('Cannot resume download: %s', filename, exc_info=True)
                 tfp = None
@@ -282,47 +267,49 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
         # Fix a problem with bad URLs that are not encoded correctly (bug 549)
         url = url.translate(self.ESCAPE_CHARS)
 
-        fp = self.open(url, data)
-        headers = fp.info()
+        session = self.init_session()
+        with session.get(url,
+                         headers=headers,
+                         stream=True,
+                         auth=auth,
+                         timeout=gpodder.SOCKET_TIMEOUT) as resp:
+            try:
+                resp.raise_for_status()
+            except HTTPError as e:
+                raise gPodderDownloadHTTPError(url, resp.status_code, str(e))
 
-        if current_size > 0:
-            # We told the server to resume - see if she agrees
-            # See RFC2616 (206 Partial Content + Section 14.16)
-            # XXX check status code here, too...
-            range = ContentRange.parse(headers.get('content-range', ''))
-            if range is None or range.start != current_size:
-                # Ok, that did not work. Reset the download
-                # TODO: seek and truncate if content-range differs from request
-                tfp.close()
-                tfp = open(filename, 'wb')
-                current_size = 0
-                logger.warn('Cannot resume: Invalid Content-Range (RFC2616).')
+            headers = resp.headers
 
-        result = headers, fp.geturl()
-        bs = 1024 * 8
-        size = -1
-        read = current_size
-        blocknum = current_size // bs
-        if reporthook:
-            if "content-length" in headers:
-                size = int(headers['Content-Length']) + current_size
-            reporthook(blocknum, bs, size)
-        while read < size or size == -1:
-            if size == -1:
-                block = fp.read(bs)
-            else:
-                block = fp.read(min(size - read, bs))
-            if len(block) == 0:
-                break
-            read += len(block)
-            tfp.write(block)
-            blocknum += 1
+            if current_size > 0:
+                # We told the server to resume - see if she agrees
+                # See RFC2616 (206 Partial Content + Section 14.16)
+                # XXX check status code here, too...
+                range = ContentRange.parse(headers.get('content-range', ''))
+                if range is None or range.start != current_size:
+                    # Ok, that did not work. Reset the download
+                    # TODO: seek and truncate if content-range differs from request
+                    tfp.close()
+                    tfp = open(filename, 'wb')
+                    current_size = 0
+                    logger.warn('Cannot resume: Invalid Content-Range (RFC2616).')
+
+            result = headers, resp.url
+            bs = 1024 * 8
+            size = -1
+            read = current_size
+            blocknum = current_size // bs
             if reporthook:
+                if "content-length" in headers:
+                    size = int(headers['content-length']) + current_size
                 reporthook(blocknum, bs, size)
-        fp.close()
-        tfp.close()
-        del fp
-        del tfp
+            for block in resp.iter_content(bs):
+                read += len(block)
+                tfp.write(block)
+                blocknum += 1
+                if reporthook:
+                    reporthook(blocknum, bs, size)
+            tfp.close()
+            del tfp
 
         # raise exception if actual size does not match content-length header
         if size >= 0 and read < size:
@@ -333,18 +320,52 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
 
 # end code based on urllib.py
 
-    def prompt_user_passwd(self, host, realm):
-        # Keep track of authentication attempts, fail after the third one
-        self._auth_retry_counter += 1
-        if self._auth_retry_counter > 3:
-            raise AuthenticationError(_('Wrong username/password'))
 
-        if self.channel.auth_username or self.channel.auth_password:
-            logger.debug('Authenticating as "%s" to "%s" for realm "%s".',
-                    self.channel.auth_username, host, realm)
-            return (self.channel.auth_username, self.channel.auth_password)
+class DefaultDownload(CustomDownload):
+    def __init__(self, config, episode, url):
+        self._config = config
+        self.__episode = episode
+        self._url = url
 
-        return (None, None)
+    def retrieve_resume(self, tempname, reporthook):
+        url = self._url
+        logger.info("Downloading %s", url)
+        max_retries = max(0, self._config.auto.retries)
+        downloader = DownloadURLOpener(self.__episode.channel, max_retries=max_retries)
+
+        # Retry the download on incomplete download (other retries are done by the Retry strategy)
+        for retry in range(max_retries + 1):
+            if retry > 0:
+                logger.info('Retrying download of %s (%d)', url, retry)
+                time.sleep(1)
+
+            try:
+                headers, real_url = downloader.retrieve_resume(url,
+                    tempname, reporthook=reporthook)
+                # If we arrive here, the download was successful
+                break
+            except urllib.error.ContentTooShortError as ctse:
+                if retry < max_retries:
+                    logger.info('Content too short: %s - will retry.',
+                            url)
+                    continue
+                raise
+        return (headers, real_url)
+
+
+class DefaultDownloader(CustomDownloader):
+    @staticmethod
+    def custom_downloader(config, episode):
+        url = episode.url
+        # Resolve URL and start downloading the episode
+        res = registry.download_url.resolve(config, None, episode, False)
+        if res:
+            url = res
+        if url == episode.url:
+            # don't modify custom urls (#635 - vimeo breaks if * is unescaped)
+            url = url.strip()
+            url = util.iri_to_url(url)
+        return DefaultDownload(config, episode, url)
 
 
 class DownloadQueueWorker(object):
@@ -572,6 +593,15 @@ class DownloadTask(object):
 
     episode = property(fget=__get_episode)
 
+    def __get_downloader(self):
+        return self.__downloader
+
+    def __set_downloader(self, downloader):
+        # modifying the downloader will only have effect before the download is started
+        self.__downloader = downloader
+
+    downloader = property(fget=__get_downloader, fset=__set_downloader)
+
     def cancel(self):
         if self.status in (self.DOWNLOADING, self.QUEUED):
             self.status = self.CANCELLED
@@ -580,13 +610,15 @@ class DownloadTask(object):
         if self.status != self.DONE:
             util.delete_file(self.tempname)
 
-    def __init__(self, episode, config):
+    def __init__(self, episode, config, downloader=None):
         assert episode.download_task is None
         self.__status = DownloadTask.INIT
         self.__activity = DownloadTask.ACTIVITY_DOWNLOAD
         self.__status_changed = True
         self.__episode = episode
         self._config = config
+        # specify a custom downloader to be used for this download
+        self.__downloader = downloader
 
         # Create the target filename and save it in the database
         self.filename = self.__episode.local_filename(create=True)
@@ -729,6 +761,7 @@ class DownloadTask(object):
             util.delete_file(self.tempname)
             self.progress = 0.0
             self.speed = 0.0
+            self.recycle()
             return False
 
         # We only start this download if its status is "downloading"
@@ -739,51 +772,28 @@ class DownloadTask(object):
         self.status = DownloadTask.DOWNLOADING
         self._notification_shown = False
 
+        # Restore a reference to this task in the episode
+        # when running a recycled task following a pause or failed
+        # see #649
+        if not self.episode.download_task:
+            self.episode.download_task = self
+
+        url = self.__episode.url
         try:
-            # Resolve URL and start downloading the episode
-            fmt_ids = youtube.get_fmt_ids(self._config.youtube)
-            url = youtube.get_real_download_url(self.__episode.url, fmt_ids)
-            url = vimeo.get_real_download_url(url, self._config.vimeo.fileformat)
-            url = escapist_videos.get_real_download_url(url)
-            url = url.strip()
+            if url == '':
+                raise DownloadNoURLException()
 
-            url = util.iri_to_url(url)
+            if self.downloader:
+                downloader = self.downloader.custom_downloader(self._config, self.episode)
+            else:
+                downloader = registry.custom_downloader.resolve(self._config, None, self.episode)
 
-            logger.info("Downloading %s", url)
-            downloader = DownloadURLOpener(self.__episode.channel)
+            if downloader:
+                logger.info('Downloading %s with %s', url, downloader)
+            else:
+                downloader = DefaultDownloader.custom_downloader(self._config, self.episode)
 
-            # HTTP Status codes for which we retry the download
-            retry_codes = (408, 418, 504, 598, 599)
-            max_retries = max(0, self._config.auto.retries)
-
-            # Retry the download on timeout (bug 1013)
-            for retry in range(max_retries + 1):
-                if retry > 0:
-                    logger.info('Retrying download of %s (%d)', url, retry)
-                    time.sleep(1)
-
-                try:
-                    headers, real_url = downloader.retrieve_resume(url,
-                        self.tempname, reporthook=self.status_updated)
-                    # If we arrive here, the download was successful
-                    break
-                except urllib.error.ContentTooShortError as ctse:
-                    if retry < max_retries:
-                        logger.info('Content too short: %s - will retry.',
-                                url)
-                        continue
-                    raise
-                except socket.timeout as tmout:
-                    if retry < max_retries:
-                        logger.info('Socket timeout: %s - will retry.', url)
-                        continue
-                    raise
-                except gPodderDownloadHTTPError as http:
-                    if retry < max_retries and http.error_code in retry_codes:
-                        logger.info('HTTP error %d: %s - will retry.',
-                                http.error_code, url)
-                        continue
-                    raise
+            headers, real_url = downloader.retrieve_resume(self.tempname, self.status_updated)
 
             new_mimetype = headers.get('content-type', self.__episode.mime_type)
             old_mimetype = self.__episode.mime_type
@@ -792,7 +802,8 @@ class DownloadTask(object):
                 logger.info('Updating mime type: %s => %s', old_mimetype, new_mimetype)
                 old_extension = self.__episode.extension()
                 self.__episode.mime_type = new_mimetype
-                new_extension = self.__episode.extension()
+                # don't call local_filename because we'll get the old download name
+                new_extension = self.__episode.extension(may_call_local_filename=False)
 
                 # If the desired filename extension changed due to the new
                 # mimetype, we force an update of the local filename to fix the
@@ -815,7 +826,7 @@ class DownloadTask(object):
                             real_url, os.path.basename(self.filename))
 
             # Look at the Content-disposition header; use if if available
-            disposition_filename = get_header_param(headers, 'filename', 'content-disposition')
+            disposition_filename = util.get_header_param(headers, 'filename', 'content-disposition')
 
             # Some servers do send the content-disposition header, but provide
             # an empty filename, resulting in an empty string here (bug 1440)
@@ -848,9 +859,27 @@ class DownloadTask(object):
                 util.delete_file(self.tempname)
                 self.progress = 0.0
                 self.speed = 0.0
+        except DownloadNoURLException:
+            self.status = DownloadTask.FAILED
+            self.error_message = _('Episode has no URL to download')
         except urllib.error.ContentTooShortError as ctse:
             self.status = DownloadTask.FAILED
             self.error_message = _('Missing content from server')
+        except ConnectionError as ce:
+            # special case request exception
+            self.status = DownloadTask.FAILED
+            logger.error('Download failed: %s', str(ce), exc_info=True)
+            d = {'host': ce.args[0].pool.host, 'port': ce.args[0].pool.port}
+            self.error_message = _("Couldn't connect to server %(host)s:%(port)s" % d)
+        except RequestException as re:
+            # extract MaxRetryError to shorten the exception message
+            if isinstance(re.args[0], MaxRetryError):
+                re = re.args[0]
+            logger.error('%s while downloading "%s"', str(re),
+                    self.__episode.title, exc_info=True)
+            self.status = DownloadTask.FAILED
+            d = {'error': str(re)}
+            self.error_message = _('Request Error: %(error)s') % d
         except IOError as ioe:
             logger.error('%s while downloading "%s": %s', ioe.strerror,
                     self.__episode.title, ioe.filename, exc_info=True)
@@ -869,6 +898,13 @@ class DownloadTask(object):
             logger.error('Download failed: %s', str(e), exc_info=True)
             self.error_message = _('Error: %s') % (str(e),)
 
+        if self.status == DownloadTask.FAILED:
+            self.__episode._download_error = self.error_message
+
+            # Delete empty partial files, they prevent streaming after a download failure (live stream)
+            if util.calculate_size(self.filename) == 0:
+                util.delete_file(self.tempname)
+
         if self.status == DownloadTask.DOWNLOADING:
             # Everything went well - we're done
             self.status = DownloadTask.DONE
