@@ -132,6 +132,32 @@ class YouTubeVideoFormatListModel(Gtk.ListStore):
         self._config.youtube.preferred_fmt_id = self[index][self.C_ID]
 
 
+class YouTubeVideoHLSFormatListModel(Gtk.ListStore):
+    C_CAPTION, C_ID = list(range(2))
+
+    def __init__(self, config):
+        Gtk.ListStore.__init__(self, str, int)
+        self._config = config
+
+        if self._config.youtube.preferred_hls_fmt_ids:
+            caption = _('Custom (%(format_ids)s)') % {
+                'format_ids': ', '.join(str(x) for x in self._config.youtube.preferred_hls_fmt_ids),
+            }
+            self.append((caption, 0))
+
+        for id, (fmt_id, path, description) in youtube.hls_formats:
+            self.append((description, id))
+
+    def get_index(self):
+        for index, row in enumerate(self):
+            if self._config.youtube.preferred_hls_fmt_id == row[self.C_ID]:
+                return index
+        return 0
+
+    def set_index(self, index):
+        self._config.youtube.preferred_hls_fmt_id = self[index][self.C_ID]
+
+
 class VimeoVideoFormatListModel(Gtk.ListStore):
     C_CAPTION, C_ID = list(range(2))
 
@@ -170,12 +196,12 @@ class gPodderPreferences(BuilderWidget):
 
         self.audio_player_model = self.user_apps_reader.get_model('audio')
         self.combo_audio_player_app.set_model(self.audio_player_model)
-        index = self.audio_player_model.get_index(self._config.player)
+        index = self.audio_player_model.get_index(self._config.player.audio)
         self.combo_audio_player_app.set_active(index)
 
         self.video_player_model = self.user_apps_reader.get_model('video')
         self.combo_video_player_app.set_model(self.video_player_model)
-        index = self.video_player_model.get_index(self._config.videoplayer)
+        index = self.video_player_model.get_index(self._config.player.video)
         self.combo_video_player_app.set_active(index)
 
         self.preferred_youtube_format_model = YouTubeVideoFormatListModel(self._config)
@@ -185,6 +211,13 @@ class gPodderPreferences(BuilderWidget):
         self.combobox_preferred_youtube_format.add_attribute(cellrenderer, 'text', self.preferred_youtube_format_model.C_CAPTION)
         self.combobox_preferred_youtube_format.set_active(self.preferred_youtube_format_model.get_index())
 
+        self.preferred_youtube_hls_format_model = YouTubeVideoHLSFormatListModel(self._config)
+        self.combobox_preferred_youtube_hls_format.set_model(self.preferred_youtube_hls_format_model)
+        cellrenderer = Gtk.CellRendererText()
+        self.combobox_preferred_youtube_hls_format.pack_start(cellrenderer, True)
+        self.combobox_preferred_youtube_hls_format.add_attribute(cellrenderer, 'text', self.preferred_youtube_hls_format_model.C_CAPTION)
+        self.combobox_preferred_youtube_hls_format.set_active(self.preferred_youtube_hls_format_model.get_index())
+
         self.preferred_vimeo_format_model = VimeoVideoFormatListModel(self._config)
         self.combobox_preferred_vimeo_format.set_model(self.preferred_vimeo_format_model)
         cellrenderer = Gtk.CellRendererText()
@@ -221,6 +254,9 @@ class gPodderPreferences(BuilderWidget):
         self.combo_auto_download.add_attribute(cellrenderer, 'text', NewEpisodeActionList.C_CAPTION)
         self.combo_auto_download.set_active(self.auto_download_model.get_index())
 
+        self._config.connect_gtk_togglebutton('check_connection',
+                                              self.checkbutton_check_connection)
+
         if self._config.auto_remove_played_episodes:
             adjustment_expiration = self.hscale_expiration.get_adjustment()
             if self._config.episode_old_age > adjustment_expiration.get_upper():
@@ -451,16 +487,20 @@ class gPodderPreferences(BuilderWidget):
 
     def on_combo_audio_player_app_changed(self, widget):
         index = self.combo_audio_player_app.get_active()
-        self._config.player = self.audio_player_model.get_command(index)
+        self._config.player.audio = self.audio_player_model.get_command(index)
 
     def on_combo_video_player_app_changed(self, widget):
         index = self.combo_video_player_app.get_active()
-        self._config.videoplayer = self.video_player_model.get_command(index)
+        self._config.player.video = self.video_player_model.get_command(index)
 
     def on_combobox_preferred_youtube_format_changed(self, widget):
         index = self.combobox_preferred_youtube_format.get_active()
         self.preferred_youtube_format_model.set_index(index)
 
+    def on_combobox_preferred_youtube_hls_format_changed(self, widget):
+        index = self.combobox_preferred_youtube_hls_format.get_active()
+        self.preferred_youtube_hls_format_model.set_index(index)
+
     def on_combobox_preferred_vimeo_format_changed(self, widget):
         index = self.combobox_preferred_vimeo_format.get_active()
         self.preferred_vimeo_format_model.set_index(index)
@@ -468,21 +508,21 @@ class gPodderPreferences(BuilderWidget):
     def on_button_audio_player_clicked(self, widget):
         result = self.show_text_edit_dialog(_('Configure audio player'),
                                             _('Command:'),
-                                            self._config.player)
+                                            self._config.player.audio)
 
         if result:
-            self._config.player = result
-            index = self.audio_player_model.get_index(self._config.player)
+            self._config.player.audio = result
+            index = self.audio_player_model.get_index(self._config.player.audio)
             self.combo_audio_player_app.set_active(index)
 
     def on_button_video_player_clicked(self, widget):
         result = self.show_text_edit_dialog(_('Configure video player'),
                                             _('Command:'),
-                                            self._config.videoplayer)
+                                            self._config.player.video)
 
         if result:
-            self._config.videoplayer = result
-            index = self.video_player_model.get_index(self._config.videoplayer)
+            self._config.player.video = result
+            index = self.video_player_model.get_index(self._config.player.video)
             self.combo_video_player_app.set_active(index)
 
     def format_update_interval_value(self, scale, value):
